@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * MainWindow.cc
- * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2019 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -64,6 +64,14 @@
 static Glib::Quark notificationHandleKey("handle");
 
 void MainWindow::signalHandler(int sig) {
+	signalHandlerExec(sig, false);
+}
+
+void MainWindow::tesseractCrash(int signal) {
+	signalHandlerExec(signal, true);
+}
+
+void MainWindow::signalHandlerExec(int sig, bool tesseractCrash) {
 	std::signal(sig, nullptr);
 	std::string filename;
 	if(MAIN->getOutputEditor() && MAIN->getOutputEditor()->getModified()) {
@@ -76,7 +84,7 @@ void MainWindow::signalHandler(int sig) {
 		MAIN->getOutputEditor()->save(filename);
 	}
 	Glib::Pid pid;
-	Glib::spawn_async("", std::vector<std::string> {pkgExePath, "crashhandle", Glib::ustring::compose("%1", getpid()), filename}, Glib::SPAWN_DO_NOT_REAP_CHILD, sigc::slot<void>(), &pid);
+	Glib::spawn_async("", std::vector<std::string> {pkgExePath, "crashhandle", Glib::ustring::compose("%1", getpid()), Glib::ustring::compose("%1", tesseractCrash), filename}, Glib::SPAWN_DO_NOT_REAP_CHILD, sigc::slot<void>(), &pid);
 #ifdef G_OS_WIN32
 	WaitForSingleObject((HANDLE)pid, 0);
 #elif __linux__
@@ -301,17 +309,21 @@ void MainWindow::showAbout() {
 
 void MainWindow::showHelp(const std::string& chapter) {
 #ifdef G_OS_WIN32
-	std::string manualDir = Glib::build_filename(pkgDir, "share", "doc", "gimagereader");
+	// Always use relative path on Windows
+	std::string manualDirPath;
 	char* locale = g_win32_getlocale();
 	std::string language(locale, 2);
 	g_free(locale);
 #else
-	std::string manualDir = MANUAL_DIR;
+	std::string manualDirPath = MANUAL_DIR;
 	std::string language = Glib::getenv("LANG").substr(0, 2);
 #endif
-	std::string manualFile = Utils::make_absolute_path(Glib::build_filename(manualDir, Glib::ustring::compose("manual-%1.html", language)), Glib::get_current_dir());
+	if(manualDirPath.empty()) {
+		manualDirPath = Glib::build_filename(pkgDir, "share", "doc", "gimagereader");
+	}
+	std::string manualFile = Utils::make_absolute_path(Glib::build_filename(manualDirPath, Glib::ustring::compose("manual-%1.html", language)), Glib::get_current_dir());
 	if(!Glib::file_test(manualFile, Glib::FILE_TEST_EXISTS)) {
-		manualFile = Utils::make_absolute_path(Glib::build_filename(manualDir, "manual.html"), Glib::get_current_dir());
+		manualFile = Utils::make_absolute_path(Glib::build_filename(manualDirPath, "manual.html"), Glib::get_current_dir());
 	}
 	Utils::openUri(Glib::filename_to_uri(manualFile) + chapter);
 }
