@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * OutputEditorText.cc
- * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2019 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -316,7 +316,7 @@ bool OutputEditorText::getModified() const {
 	return m_textBuffer->get_modified();
 }
 
-void OutputEditorText::onVisibilityChanged(bool /*visibile*/) {
+void OutputEditorText::onVisibilityChanged(bool /*visible*/) {
 	m_searchFrame->hideSubstitutionsManager();
 }
 
