@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * Recognizer.cc
- * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2019 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -106,11 +106,17 @@ Recognizer::Recognizer(const Ui::MainWindow& _ui)
 		Utils::clear_error_state(ui.entryPageRange);
 		return false;
 	});
+	CONNECT(ui.radioButtonBlacklist, toggled, [this] { ui.entryBlacklist->set_sensitive(ui.radioButtonBlacklist->get_active()); });
+	CONNECT(ui.radioButtonWhitelist, toggled, [this] { ui.entryWhitelist->set_sensitive(ui.radioButtonWhitelist->get_active()); });
 
 	ADD_SETTING(VarSetting<Glib::ustring>("language"));
 	ADD_SETTING(ComboSetting("ocrregionstrategy", ui.comboPageRangeRegions));
 	ADD_SETTING(SwitchSettingT<Gtk::CheckButton>("ocraddsourcefilename", ui.checkPageRangePrependFile));
 	ADD_SETTING(SwitchSettingT<Gtk::CheckButton>("ocraddsourcepage", ui.checkPageRangePrependPage));
+	ADD_SETTING(EntrySetting("ocrcharwhitelist", ui.entryWhitelist));
+	ADD_SETTING(EntrySetting("ocrcharblacklist", ui.entryBlacklist));
+	ADD_SETTING(SwitchSettingT<Gtk::RadioButton>("ocrblacklistenabled", ui.radioButtonBlacklist));
+	ADD_SETTING(SwitchSettingT<Gtk::RadioButton>("ocrwhitelistenabled", ui.radioButtonWhitelist));
 	ADD_SETTING(VarSetting<int>("psm"));
 }
 
@@ -134,55 +140,16 @@ std::vector<Glib::ustring> Recognizer::getAvailableLanguages() const {
 	return result;
 }
 
-static int g_pipe[2];
-static jmp_buf g_restore_point;
-
-static void tessCrashHandler(int /*signal*/) {
-	fflush(stderr);
-	char buf[1025];
-	int bytesRead = 0;
-	Glib::ustring captured;
-	do {
-		if((bytesRead = read(g_pipe[0], buf, sizeof(buf) - 1)) > 0) {
-			buf[bytesRead] = 0;
-			captured += buf;
-		}
-	} while(bytesRead == sizeof(buf) - 1);
-	tesseract::TessBaseAPI tess;
-	Glib::ustring errMsg = Glib::ustring::compose(_("Tesseract crashed with the following message:\n\n"
-	                       "%1\n\n"
-	                       "This typically happens for one of the following reasons:\n"
-	                       "- Outdated traineddata files are used.\n"
-	                       "- Auxiliary language data files are missing.\n"
-	                       "- Corrupt language data files.\n\n"
-	                       "Make sure your language data files are valid and compatible with tesseract %2."), captured, tess.Version());
-	Utils::message_dialog(Gtk::MESSAGE_ERROR, _("Error"), errMsg);
-	longjmp(g_restore_point, SIGSEGV);
-}
-
 tesseract::TessBaseAPI Recognizer::initTesseract(const char* language, bool* ok) const {
+	// unfortunately tesseract creates deliberate segfaults when an error occurs
+	std::signal(SIGABRT, MainWindow::tesseractCrash);
 	std::string current = setlocale(LC_ALL, NULL);
 	setlocale(LC_ALL, "C");
 	tesseract::TessBaseAPI tess;
-	// unfortunately tesseract creates deliberate segfaults when an error occurs
-	std::signal(SIGSEGV, tessCrashHandler);
-	pipe(g_pipe);
-	fflush(stderr);
-	int oldstderr = dup(fileno(stderr));
-	dup2(g_pipe[1], fileno(stderr));
-	int ret = -1;
-	int fault_code = setjmp(g_restore_point);
-	if(fault_code == 0) {
-		ret = tess.Init(nullptr, language);
-	} else {
-		ret = -1;
-	}
-	dup2(oldstderr, fileno(stderr));
+	int ret = tess.Init(nullptr, language);
 	std::signal(SIGSEGV, MainWindow::signalHandler);
 	setlocale(LC_ALL, current.c_str());
 
-	close(g_pipe[0]);
-	close(g_pipe[1]);
 	if(ok) {
 		*ok = ret != -1;
 	}
@@ -343,6 +310,10 @@ void Recognizer::updateLanguagesMenu() {
 	psmItem->set_submenu(*psmMenu);
 	ui.menuLanguages->append(*psmItem);
 
+	Gtk::MenuItem* charlistItem = Gtk::manage(new Gtk::MenuItem(_("Character whitelist / blacklist...")));
+	CONNECT(charlistItem, activate, [this] { manageCharaterLists(); });
+	ui.menuLanguages->append(*charlistItem);
+
 	// Add installer item
 	ui.menuLanguages->append(*Gtk::manage(new Gtk::SeparatorMenuItem()));
 	Gtk::MenuItem* manageItem = Gtk::manage(new Gtk::MenuItem(_("Manage languages...")));
@@ -442,6 +413,11 @@ std::vector<int> Recognizer::selectPages(bool& autodetectLayout) {
 	return pages;
 }
 
+void Recognizer::manageCharaterLists() {
+	ui.dialogCharacterLists->run();
+	ui.dialogCharacterLists->hide();
+}
+
 void Recognizer::recognizeButtonClicked() {
 	int nPages = MAIN->getDisplayer()->getNPages();
 	if(nPages == 1) {
@@ -470,6 +446,12 @@ void Recognizer::recognize(const std::vector<int>& pages, bool autodetectLayout)
 	if(ok) {
 		Glib::ustring failed;
 		tess.SetPageSegMode(static_cast<tesseract::PageSegMode>(m_currentPsmMode));
+		if(ui.radioButtonWhitelist->get_active()) {
+			tess.SetVariable("tessedit_char_whitelist", ui.entryWhitelist->get_text().c_str());
+		}
+		if(ui.radioButtonBlacklist->get_active()) {
+			tess.SetVariable("tessedit_char_blacklist", ui.entryBlacklist->get_text().c_str());
+		}
 		OutputEditor::ReadSessionData* readSessionData = MAIN->getOutputEditor()->initRead(tess);
 		ProgressMonitor monitor(pages.size());
 		MAIN->showProgress(&monitor);
