@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * TessdataManager.hh
- * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2019 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -160,6 +160,9 @@ bool TessdataManager::fetchLanguageList(Glib::ustring& messages) {
 			JsonNode* value = static_cast<JsonNode*>(l->data);
 			JsonObject* treeObj = json_node_get_object(value);
 			Glib::ustring fileName = json_object_get_string_member(treeObj, "name");
+			if(!json_object_get_string_member(treeObj, "download_url")) {
+				continue;
+			}
 			Glib::ustring url = json_object_get_string_member(treeObj, "download_url");
 			Glib::ustring subdir;
 			// If filename starts with upper case letter, it is a script
