@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * Utils.cc
- * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2019 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -281,6 +281,22 @@ Glib::ustring Utils::string_html_escape(const Glib::ustring& str) {
 	return result;
 }
 
+std::vector<std::pair<Glib::ustring, int>> Utils::string_split_pos(const Glib::ustring& str, const Glib::RefPtr<Glib::Regex>& splitRe) {
+	int strpos = 0;
+	Glib::MatchInfo info;
+	std::vector<std::pair<Glib::ustring, int>> result;
+	while(splitRe->match(str, strpos, info)) {
+		int start = 0, end = 0;
+		info.fetch_pos(0, start, end);
+		result.push_back(std::make_pair(str.substr(strpos, start - strpos), strpos));
+		strpos = end;
+	}
+	if(strpos < str.length()) {
+		result.push_back(std::make_pair(str.substr(strpos), strpos));
+	}
+	return result;
+}
+
 int Utils::parseInt(const Glib::ustring& str, bool* ok) {
 	static Glib::RefPtr<Glib::Regex> nrRegEx = Glib::Regex::create("^\\d+$");
 	bool match = nrRegEx->match(str);
@@ -366,10 +382,7 @@ Glib::RefPtr<Glib::ByteArray> Utils::download(const std::string& url, Glib::ustr
 
 Glib::ustring Utils::getSpellingLanguage(const Glib::ustring& lang) {
 	// If it is already a valid code, return it
-	if(lang.length() == 2) {
-		return lang;
-	}
-	static Glib::RefPtr<Glib::Regex> langPattern = Glib::Regex::create("^[a-z]{2}_[A-Z]{2}");
+	static Glib::RefPtr<Glib::Regex> langPattern = Glib::Regex::create("^[a-z]{2,}(_[A-Z]{2,})?$");
 	if(langPattern->match(lang)) {
 		return lang;
 	}
