@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * HOCROdtExporter.cc
- * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2019 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -401,8 +401,8 @@ void HOCROdtExporter::writeFontFaceDecls(std::set<Glib::ustring>& families, cons
 			families.insert(fontFamily);
 		}
 	} else {
-		for(const HOCRItem* item : item->children()) {
-			writeFontFaceDecls(families, item, parentEl);
+		for(const HOCRItem* child : item->children()) {
+			writeFontFaceDecls(families, child, parentEl);
 		}
 	}
 }
@@ -439,8 +439,8 @@ void HOCROdtExporter::writeFontStyles(std::map<Glib::ustring, std::map<double, G
 			styles[fontKey].insert(std::make_pair(item->fontSize(), styleName));
 		}
 	} else {
-		for(const HOCRItem* item : item->children()) {
-			writeFontStyles(styles, item, parentEl, counter);
+		for(const HOCRItem* child : item->children()) {
+			writeFontStyles(styles, child, parentEl, counter);
 		}
 	}
 }
