@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * HOCRPdfExporter.cc
- * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2019 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -444,6 +444,7 @@ HOCRPdfExporter::HOCRPdfExporter(const Glib::RefPtr<HOCRDocument>& hocrdocument,
 	ADD_SETTING(SpinSetting("pdffontsize", ui.spinOverridefontsize));
 	ADD_SETTING(SpinSetting("pdffontscale", ui.spinFontscale));
 	ADD_SETTING(SwitchSettingT<Gtk::CheckButton>("pdfuniformizelinespacing", ui.checkboxUniformlinespacing));
+	ADD_SETTING(SwitchSettingT<Gtk::CheckButton>("pdfsanitizehyphens", ui.checkboxSanitizeHyphens));
 	ADD_SETTING(SpinSetting("pdfpreservespaces", ui.spinPreserve));
 	ADD_SETTING(SwitchSettingT<Gtk::CheckButton>("pdfpreview", ui.checkboxPreview));
 	ADD_SETTING(ComboSetting("pdfexportpapersize", ui.comboPaperFormat));
@@ -560,13 +561,27 @@ bool HOCRPdfExporter::run(std::string& filebasename) {
 			const HOCRPage* page = m_hocrdocument->page(i);
 			if(page->isEnabled()) {
 				Geometry::Rectangle bbox = page->bbox();
+				Glib::ustring sourceFile = page->sourceFile();
+				// If the source file is an image, its "resolution" is actually just the scale factor that was used for recognizing.
+				bool isImage = Glib::ustring(sourceFile.substr(sourceFile.length() - 4)).lowercase() != ".pdf" && Glib::ustring(sourceFile.substr(sourceFile.length() - 5)).lowercase() != ".djvu";
 				int sourceDpi = page->resolution();
+				int sourceScale = sourceDpi;
+				if(isImage) {
+					sourceDpi *= ui.spinBoxPaperSizeDpi->get_value_as_int() / 100;
+				}
+				// [pt] = 72 * [in]
+				// [in] = 1 / dpi * [px]
+				// => [pt] = 72 / dpi * [px]
+				double px2pt = (72.0 / sourceDpi);
+				double imgScale = double(outputDpi) / sourceDpi;
+
 				bool success = false;
-				Utils::runInMainThreadBlocking([&] { success = setSource(page->sourceFile(), page->pageNr(), outputDpi, page->angle()); });
+				if(isImage) {
+					Utils::runInMainThreadBlocking([&] { success = setSource(page->sourceFile(), page->pageNr(), sourceScale * imgScale, page->angle()); });
+				} else {
+					Utils::runInMainThreadBlocking([&] { success = setSource(page->sourceFile(), page->pageNr(), outputDpi, page->angle()); });
+				}
 				if(success) {
-					// [pt] = 72 * [in]
-					// [in] = 1 / dpi * [px]
-					// => [pt] = 72 / dpi * [px]
 					double px2pt = (72.0 / sourceDpi);
 					double imgScale = double(outputDpi) / sourceDpi;
 					if(paperSize == "source") {
@@ -578,7 +593,7 @@ bool HOCRPdfExporter::run(std::string& filebasename) {
 					if(!painter->createPage(pageWidth, pageHeight, offsetX, offsetY, errMsg)) {
 						return false;
 					}
-					printChildren(*painter, page, pdfSettings, px2pt, imgScale, true);
+					printChildren(*painter, page, pdfSettings, px2pt, imgScale, double(sourceScale) / sourceDpi, true);
 					if(pdfSettings.overlay) {
 						Geometry::Rectangle scaledRect(imgScale * bbox.x, imgScale * bbox.y, imgScale * bbox.width, imgScale * bbox.height);
 						Geometry::Rectangle printRect(bbox.x * px2pt, bbox.y * px2pt, bbox.width * px2pt, bbox.height * px2pt);
@@ -586,7 +601,7 @@ bool HOCRPdfExporter::run(std::string& filebasename) {
 						Utils::runInMainThreadBlocking([&] { selection = getSelection(scaledRect); });
 						painter->drawImage(printRect, selection, pdfSettings);
 					}
-					Utils::runInMainThreadBlocking([&] { setSource(page->sourceFile(), page->pageNr(), sourceDpi, page->angle()); });
+					Utils::runInMainThreadBlocking([&] { setSource(page->sourceFile(), page->pageNr(), sourceScale, page->angle()); });
 					painter->finishPage();
 				} else {
 					errMsg = Glib::ustring::compose(_("Failed to render page %1"), page->title());
@@ -683,13 +698,17 @@ HOCRPdfExporter::PDFSettings HOCRPdfExporter::getPdfSettings() const {
 	pdfSettings.preserveSpaceWidth = ui.spinPreserve->get_value();
 	pdfSettings.overlay = ui.comboMode->get_active_row_number() == 1;
 	pdfSettings.detectedFontScaling = ui.spinFontscale->get_value() / 100.;
+	pdfSettings.sanitizeHyphens = ui.checkboxSanitizeHyphens->get_active();
 	return pdfSettings;
 }
 
-void HOCRPdfExporter::printChildren(PDFPainter& painter, const HOCRItem* item, const PDFSettings& pdfSettings, double px2pu/*pixels to printer units*/, double imgScale, bool inThread) {
+void HOCRPdfExporter::printChildren(PDFPainter& painter, const HOCRItem* item, const PDFSettings& pdfSettings, double px2pu/*pixels to printer units*/, double imgScale, double fontScale, bool inThread) {
 	if(!item->isEnabled()) {
 		return;
 	}
+	if(pdfSettings.fontSize != -1) {
+		painter.setFontSize(pdfSettings.fontSize * fontScale);
+	}
 	Glib::ustring itemClass = item->itemClass();
 	Geometry::Rectangle itemRect = item->bbox();
 	int childCount = item->children().size();
@@ -709,7 +728,7 @@ void HOCRPdfExporter::printChildren(PDFPainter& painter, const HOCRItem* item, c
 				Geometry::Rectangle wordRect = wordItem->bbox();
 				painter.setFontFamily(pdfSettings.fontFamily.empty() ? wordItem->fontFamily() : pdfSettings.fontFamily, wordItem->fontBold(), wordItem->fontItalic());
 				if(pdfSettings.fontSize == -1) {
-					painter.setFontSize(wordItem->fontSize() * pdfSettings.detectedFontScaling);
+					painter.setFontSize(wordItem->fontSize() * pdfSettings.detectedFontScaling * fontScale);
 				}
 				// If distance from previous word is large, keep the space
 				if(wordRect.x - prevWordRight > pdfSettings.preserveSpaceWidth * painter.getAverageCharWidth() / px2pu) {
@@ -717,6 +736,9 @@ void HOCRPdfExporter::printChildren(PDFPainter& painter, const HOCRItem* item, c
 				}
 				prevWordRight = wordRect.x + wordRect.width;
 				Glib::ustring text = wordItem->text();
+				if(iWord == nWords - 1 && pdfSettings.sanitizeHyphens) {
+					text = Glib::Regex::create("[-\u2014]\\s*$")->replace(text, 0, "-", static_cast<Glib::RegexMatchFlags>(0));
+				}
 				double wordBaseline = (x - itemRect.x) * baseline.first + baseline.second;
 				painter.drawText(x * px2pu, (y + wordBaseline) * px2pu, text);
 				x += painter.getTextWidth(text + " ") / px2pu;
@@ -732,10 +754,14 @@ void HOCRPdfExporter::printChildren(PDFPainter& painter, const HOCRItem* item, c
 			Geometry::Rectangle wordRect = wordItem->bbox();
 			painter.setFontFamily(pdfSettings.fontFamily.empty() ? wordItem->fontFamily() : pdfSettings.fontFamily, wordItem->fontBold(), wordItem->fontItalic());
 			if(pdfSettings.fontSize == -1) {
-				painter.setFontSize(wordItem->fontSize() * pdfSettings.detectedFontScaling);
+				painter.setFontSize(wordItem->fontSize() * pdfSettings.detectedFontScaling * fontScale);
 			}
 			double y = itemRect.y + itemRect.height + (wordRect.x + 0.5 * wordRect.width - itemRect.x) * baseline.first + baseline.second;
-			painter.drawText(wordRect.x * px2pu, y * px2pu, wordItem->text());
+			Glib::ustring text = wordItem->text();
+			if(iWord == nWords - 1 && pdfSettings.sanitizeHyphens) {
+				text = Glib::Regex::create("[-\u2014]\\s*$")->replace(text, 0, "-", static_cast<Glib::RegexMatchFlags>(0));
+			}
+			painter.drawText(wordRect.x * px2pu, y * px2pu, text);
 		}
 	} else if(itemClass == "ocr_graphic" && !pdfSettings.overlay) {
 		Geometry::Rectangle scaledItemRect(imgScale * itemRect.x, imgScale * itemRect.y, imgScale * itemRect.width, imgScale * itemRect.height);
@@ -749,7 +775,7 @@ void HOCRPdfExporter::printChildren(PDFPainter& painter, const HOCRItem* item, c
 		painter.drawImage(printRect, selection, pdfSettings);
 	} else {
 		for(int i = 0, n = item->children().size(); i < n; ++i) {
-			printChildren(painter, item->children()[i], pdfSettings, px2pu, imgScale, inThread);
+			printChildren(painter, item->children()[i], pdfSettings, px2pu, imgScale, fontScale, inThread);
 		}
 	}
 }
@@ -856,14 +882,17 @@ void HOCRPdfExporter::paperSizeChanged() {
 		ui.entryPaperHeight->set_sensitive(true);
 		ui.boxPaperSize->set_visible(true);
 		ui.boxPaperOrientation->set_visible(false);
+		ui.boxPaperSizeDpi->set_visible(false);
 	} else if(paperSize == "source") {
 		ui.entryPaperWidth->set_sensitive(true);
 		ui.entryPaperHeight->set_sensitive(true);
 		ui.boxPaperSize->set_visible(false);
 		ui.boxPaperOrientation->set_visible(false);
+		ui.boxPaperSizeDpi->set_visible(true);
 	} else {
 		ui.boxPaperSize->set_visible(true);
 		ui.boxPaperOrientation->set_visible(true);
+		ui.boxPaperSizeDpi->set_visible(false);
 		ui.entryPaperWidth->set_sensitive(false);
 		ui.entryPaperHeight->set_sensitive(false);
 		PaperSize::Unit unit = static_cast<PaperSize::Unit>(static_cast<int>((*ui.comboPaperSizeUnit->get_active())[m_sizeUnitComboCols.unit]));
@@ -914,7 +943,7 @@ void HOCRPdfExporter::importMetadataFromSource() {
 		ui.entryMetadataProducer->set_text(source->producer);
 
 		PoDoFo::EPdfVersion sourcePdfVersion = PoDoFo::EPdfVersion::ePdfVersion_1_7;
-		if(source->pdfVersionMajor == 1 && source->pdfVersionMinor >= 0 && source->pdfVersionMinor <= 7) {
+		if(source->pdfVersionMajor == 1 && source->pdfVersionMinor <= 7) {
 			sourcePdfVersion = static_cast<PoDoFo::EPdfVersion>(PoDoFo::EPdfVersion::ePdfVersion_1_0 + source->pdfVersionMinor);
 		}
 		ui.comboPdfVersion->set_active(sourcePdfVersion);
