@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * OutputEditorHOCR.cc
- * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2019 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -571,9 +571,9 @@ void OutputEditorHOCR::showItemProperties(const Gtk::TreeIter& index, const Gtk:
 	}
 
 	// ocr_class:attr_key:attr_values
-	std::map<Glib::ustring, std::map<Glib::ustring, std::set<Glib::ustring>>> occurences;
-	currentItem->getPropagatableAttributes(occurences);
-	for(auto it = occurences.begin(), itEnd = occurences.end(); it != itEnd; ++it) {
+	std::map<Glib::ustring, std::map<Glib::ustring, std::set<Glib::ustring>>> occurrences;
+	currentItem->getPropagatableAttributes(occurrences);
+	for(auto it = occurrences.begin(), itEnd = occurrences.end(); it != itEnd; ++it) {
 		Gtk::TreeIter item = m_propStore->append();
 		Glib::ustring itemClass = it->first;
 		item->set_value(m_propStoreCols.name, itemClass);
@@ -620,7 +620,7 @@ void OutputEditorHOCR::showItemProperties(const Gtk::TreeIter& index, const Gtk:
 Glib::RefPtr<Glib::Regex> OutputEditorHOCR::attributeValidator(const Glib::ustring& attribName) const {
 	static std::map<Glib::ustring, Glib::RefPtr<Glib::Regex>> validators = {
 		{"title:bbox", Glib::Regex::create("^\\d+\\s+\\d+\\s+\\d+\\s+\\d+$")},
-		{"lang", Glib::Regex::create("^[a-z]{2}(?:_[A-Z]{2})?$")},
+		{"lang", Glib::Regex::create("^[a-z]{2,}(?:_[A-Z]{2,})?$")},
 		{"title:x_fsize", Glib::Regex::create("^\\d+$")},
 		{"title:baseline", Glib::Regex::create("^[-+]?\\d+\\.?\\d*\\s[-+]?\\d+\\.?\\d*$")}
 	};
@@ -681,6 +681,8 @@ void OutputEditorHOCR::bboxDrawn(const Geometry::Rectangle& bbox, int action) {
 	if(!currentItem) {
 		return;
 	}
+	std::map<Glib::ustring, std::map<Glib::ustring, std::set<Glib::ustring>>> propAttrs;
+	currentItem->getPropagatableAttributes(propAttrs);
 	xmlpp::Element* newElement;
 	if(action == DisplayerToolHOCR::ACTION_DRAW_GRAPHIC_RECT) {
 		newElement = doc.create_root_node("div");
@@ -697,6 +699,7 @@ void OutputEditorHOCR::bboxDrawn(const Geometry::Rectangle& bbox, int action) {
 	} else if(action == DisplayerToolHOCR::ACTION_DRAW_LINE_RECT) {
 		newElement = doc.create_root_node("span");
 		newElement->set_attribute("class", "ocr_line");
+		std::set<Glib::ustring> propLineBaseline = propAttrs["ocrx_line"]["baseline"];
 		// Tesseract does as follows:
 		// row_height = x_height + ascenders - descenders
 		// font_pt_size = row_height * 72 / dpi (72 = pointsPerInch)
@@ -706,7 +709,7 @@ void OutputEditorHOCR::bboxDrawn(const Geometry::Rectangle& bbox, int action) {
 		titleAttrs["x_ascenders"] = Glib::ustring::compose("%1", 0.25 * bbox.height);
 		titleAttrs["x_descenders"] = Glib::ustring::compose("%1", 0.25 * bbox.height);
 		titleAttrs["x_size"] = Glib::ustring::compose("%1", bbox.height);
-		titleAttrs["baseline"] = Glib::ustring("0 0");
+		titleAttrs["baseline"] = propLineBaseline.size() == 1 ? *propLineBaseline.begin() : Glib::ustring("0 0");
 		newElement->set_attribute("title", HOCRItem::serializeAttrGroup(titleAttrs));
 	} else if(action == DisplayerToolHOCR::ACTION_DRAW_WORD_RECT) {
 		ui.entryAddWord->set_text("");
@@ -717,13 +720,17 @@ void OutputEditorHOCR::bboxDrawn(const Geometry::Rectangle& bbox, int action) {
 		}
 		newElement = doc.create_root_node("span");
 		newElement->set_attribute("class", "ocrx_word");
-		newElement->set_attribute("lang", m_spell.get_language());
+		std::map<Glib::ustring, std::set<Glib::ustring>> propWord = propAttrs["ocrx_word"];
+
+		newElement->set_attribute("lang", propWord["lang"].size() == 1 ? *propWord["lang"].begin() : m_spell.get_language());
 		std::map<Glib::ustring, Glib::ustring> titleAttrs;
 		titleAttrs["bbox"] = Glib::ustring::compose("%1 %2 %3 %4", bbox.x, bbox.y, bbox.x + bbox.width, bbox.y + bbox.height);
-		titleAttrs["x_font"] = "sans";
-		titleAttrs["x_fsize"] = Glib::ustring::compose("%1", std::round(bbox.height * 72. / currentItem->page()->resolution()));
+		titleAttrs["x_font"] = propWord["title:x_font"].size() == 1 ? *propWord["title:x_font"].begin() : "sans";
+		titleAttrs["x_fsize"] = propWord["title:x_fsize"].size() == 1 ? *propWord["title:x_fsize"].begin() : Glib::ustring::compose("%1", std::round(bbox.height * 72. / currentItem->page()->resolution()));
 		titleAttrs["x_wconf"] = "100";
 		newElement->set_attribute("title", HOCRItem::serializeAttrGroup(titleAttrs));
+		if (propWord["bold"].size() == 1) { newElement->set_attribute("bold", *propWord["bold"].begin()); }
+		if (propWord["italic"].size() == 1) { newElement->set_attribute("italic", *propWord["italic"].begin()); }
 		newElement->add_child_text(ui.entryAddWord->get_text());
 	} else {
 		return;
@@ -831,28 +838,22 @@ void OutputEditorHOCR::showTreeWidgetContextMenu(GdkEventButton* ev) {
 		CONNECT(addWordItem, activate, [this] { m_tool->setAction(DisplayerToolHOCR::ACTION_DRAW_WORD_RECT); });
 	} else if(itemClass == "ocrx_word") {
 		Glib::ustring prefix, suffix, trimmedWord = HOCRItem::trimmedWord(item->text(), &prefix, &suffix);
-		Glib::ustring spellLang = item->lang();
-		bool haveLanguage = true;
-		if(m_spell.get_language() != spellLang) {
-			try {
-				m_spell.set_language(spellLang);
-			} catch(const GtkSpell::Error& /*e*/) {
-				haveLanguage = false;
-			}
+		std::vector<Glib::ustring> suggestions;
+		bool valid = m_document->checkItemSpelling(index, &suggestions, 16);
+		for(const Glib::ustring& suggestion : suggestions) {
+			Glib::ustring replacement = prefix + suggestion + suffix;
+			Gtk::MenuItem* item = Gtk::manage(new Gtk::MenuItem(replacement));
+			CONNECT(item, activate, [this, replacement, index] { m_document->editItemText(index, replacement); });
+			menu.append(*item);
 		}
-		if(haveLanguage && !trimmedWord.empty()) {
-			for(const Glib::ustring& suggestion : m_spell.get_suggestions(trimmedWord)) {
-				Glib::ustring replacement = prefix + suggestion + suffix;
-				Gtk::MenuItem* item = Gtk::manage(new Gtk::MenuItem(replacement));
-				CONNECT(item, activate, [this, replacement, index] { m_document->editItemText(index, replacement); });
-				menu.append(*item);
-			}
+
+		if(!trimmedWord.empty()) {
 			if(menu.get_children().empty()) {
 				Gtk::MenuItem* item = Gtk::manage(new Gtk::MenuItem(_("No suggestions")));
 				item->set_sensitive(false);
 				menu.append(*item);
 			}
-			if(!m_spell.check_word(trimmedWord)) {
+			if(!valid) {
 				menu.append(*Gtk::manage(new Gtk::SeparatorMenuItem));
 				Gtk::MenuItem* additem = Gtk::manage(new Gtk::MenuItem(_("Add to dictionary")));
 				CONNECT(additem, activate, [this, index, trimmedWord] {
@@ -972,7 +973,10 @@ bool OutputEditorHOCR::save(const std::string& filename) {
 		Utils::message_dialog(Gtk::MESSAGE_ERROR, _("Failed to save output"), _("Check that you have writing permissions in the selected folder."));
 		return false;
 	}
+	std::string current = setlocale(LC_ALL, NULL);
+	setlocale(LC_ALL, "C");
 	tesseract::TessBaseAPI tess;
+	setlocale(LC_ALL, current.c_str());
 	Glib::ustring header = Glib::ustring::compose(
 	                           "<!DOCTYPE html>\n"
 	                           "<html>\n"
@@ -1073,7 +1077,7 @@ void OutputEditorHOCR::setLanguage(const Config::Lang& lang) {
 	m_document->setDefaultLanguage(lang.code);
 }
 
-void OutputEditorHOCR::onVisibilityChanged(bool /*visibile*/) {
+void OutputEditorHOCR::onVisibilityChanged(bool /*visible*/) {
 	m_searchFrame->hideSubstitutionsManager();
 }
 
@@ -1236,7 +1240,7 @@ void OutputEditorHOCR::drawPreview(Cairo::RefPtr<Cairo::Context> context, const
 				continue;
 			}
 			const Geometry::Rectangle& wordRect = wordItem->bbox();
-			context->select_font_face(wordItem->fontFamily(), wordItem->fontItalic() ? Cairo::FONT_SLANT_ITALIC : Cairo::FONT_SLANT_NORMAL, wordItem->fontBold() ? Cairo::FONT_WEIGHT_BOLD : Cairo::FONT_WEIGHT_NORMAL);
+			context->select_font_face(wordItem->fontFamily().empty() ? "sans" : wordItem->fontFamily(), wordItem->fontItalic() ? Cairo::FONT_SLANT_ITALIC : Cairo::FONT_SLANT_NORMAL, wordItem->fontBold() ? Cairo::FONT_WEIGHT_BOLD : Cairo::FONT_WEIGHT_NORMAL);
 			context->set_font_size(wordItem->fontSize() * (item->page()->resolution() / 72.));
 			// See https://github.com/kba/hocr-spec/issues/15
 			double y = lineRect.y + lineRect.height + (wordRect.x + 0.5 * wordRect.width - lineRect.x) * baseline.first + baseline.second;
