@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * CrashHandler.cc
- * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2019 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -21,9 +21,11 @@
 #include "CrashHandler.hh"
 #include <QPushButton>
 
-CrashHandler::CrashHandler(int pid, const QString& savefile, QWidget* parent):
+CrashHandler::CrashHandler(int pid, int tesseractCrash, const QString& savefile, QWidget* parent):
 	QDialog(parent), m_pid(pid) {
 	ui.setupUi(this);
+	ui.labelIntroTesseract->setVisible(tesseractCrash);
+	ui.labelIntro->setVisible(!tesseractCrash);
 
 	if(!savefile.isEmpty()) {
 		ui.labelAutosave->setText(_("Your work has been saved under <b>%1</b>.").arg(savefile));
