@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * MainWindow.cc
- * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2019 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -58,6 +58,14 @@
 #define CHANGELOGURL "https://raw.githubusercontent.com/manisandro/gImageReader/master/NEWS"
 
 void MainWindow::signalHandler(int signal) {
+	signalHandlerExec(signal, false);
+}
+
+void MainWindow::tesseractCrash(int signal) {
+	signalHandlerExec(signal, true);
+}
+
+void MainWindow::signalHandlerExec(int signal, bool tesseractCrash) {
 	std::signal(signal, nullptr);
 
 	QString filename;
@@ -72,7 +80,7 @@ void MainWindow::signalHandler(int signal) {
 	}
 
 	QProcess process;
-	process.start(QApplication::applicationFilePath(), QStringList() << "crashhandle" << QString::number(QApplication::applicationPid()) << filename);
+	process.start(QApplication::applicationFilePath(), QStringList() << "crashhandle" << QString::number(QApplication::applicationPid()) << QString::number(tesseractCrash) << filename);
 #ifdef Q_OS_LINUX
 	// Allow crash handler spawned debugger to attach to the crashed process
 	prctl(PR_SET_PTRACER, process.pid(), 0, 0, 0);
@@ -290,10 +298,15 @@ void MainWindow::showAbout() {
 
 void MainWindow::showHelp(const QString& chapter) {
 #ifdef Q_OS_WIN32
-	QDir manualDir(QString("%1/../share/doc/gimagereader").arg(QApplication::applicationDirPath()));
+	// Always use relative path on Windows
+	QString manualDirPath;
 #else
-	QDir manualDir(MANUAL_DIR);
+	QString manualDirPath(MANUAL_DIR);
 #endif
+	if(manualDirPath.isEmpty()) {
+		manualDirPath = QString("%1/../share/doc/gimagereader").arg(QApplication::applicationDirPath());
+	}
+	QDir manualDir(manualDirPath);
 	QString language = QLocale::system().name().left(2);
 	QString manualFile = manualDir.absoluteFilePath(QString("manual-%1.html").arg(language));
 	if(!QFile(manualFile).exists()) {
