@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * Recognizer.hh
- * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2019 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -86,6 +86,9 @@ Recognizer::Recognizer(const UI_MainWindow& _ui) :
 	m_pagesDialog = new QDialog(MAIN);
 	m_pagesDialogUi.setupUi(m_pagesDialog);
 
+	m_charListDialog = new QDialog(MAIN);
+	m_charListDialogUi.setupUi(m_charListDialog);
+
 	ui.toolButtonRecognize->setText(QString("%1\n%2").arg(m_modeLabel).arg(m_langLabel));
 	ui.menuLanguages->installEventFilter(this);
 
@@ -93,11 +96,17 @@ Recognizer::Recognizer(const UI_MainWindow& _ui) :
 	connect(currentPageAction, SIGNAL(triggered()), this, SLOT(recognizeCurrentPage()));
 	connect(multiplePagesAction, SIGNAL(triggered()), this, SLOT(recognizeMultiplePages()));
 	connect(m_pagesDialogUi.lineEditPageRange, SIGNAL(textChanged(QString)), this, SLOT(clearLineEditPageRangeStyle()));
+	connect(m_charListDialogUi.radioButtonBlacklist, SIGNAL(toggled(bool)), m_charListDialogUi.lineEditBlacklist, SLOT(setEnabled(bool)));
+	connect(m_charListDialogUi.radioButtonWhitelist, SIGNAL(toggled(bool)), m_charListDialogUi.lineEditWhitelist, SLOT(setEnabled(bool)));
 
 	ADD_SETTING(VarSetting<QString>("language", "eng:en_EN"));
 	ADD_SETTING(ComboSetting("ocrregionstrategy", m_pagesDialogUi.comboBoxRecognitionArea, 0));
 	ADD_SETTING(SwitchSetting("ocraddsourcefilename", m_pagesDialogUi.checkBoxPrependFilename));
 	ADD_SETTING(SwitchSetting("ocraddsourcepage", m_pagesDialogUi.checkBoxPrependPage));
+	ADD_SETTING(LineEditSetting("ocrcharwhitelist", m_charListDialogUi.lineEditWhitelist));
+	ADD_SETTING(LineEditSetting("ocrcharblacklist", m_charListDialogUi.lineEditBlacklist));
+	ADD_SETTING(SwitchSetting("ocrblacklistenabled", m_charListDialogUi.radioButtonBlacklist, true));
+	ADD_SETTING(SwitchSetting("ocrwhitelistenabled", m_charListDialogUi.radioButtonWhitelist, false));
 	ADD_SETTING(VarSetting<int>("psm", 6));
 }
 
@@ -121,55 +130,15 @@ QStringList Recognizer::getAvailableLanguages() const {
 	return result;
 }
 
-static int g_pipe[2];
-static jmp_buf g_restore_point;
-
-static void tessCrashHandler(int /*signal*/) {
-	fflush(stderr);
-	char buf[1025];
-	int bytesRead = 0;
-	QString captured;
-	do {
-		if((bytesRead = read(g_pipe[0], buf, sizeof(buf) - 1)) > 0) {
-			buf[bytesRead] = 0;
-			captured += buf;
-		}
-	} while(bytesRead == sizeof(buf) - 1);
-	tesseract::TessBaseAPI tess;
-	QString errMsg = QString(_("Tesseract crashed with the following message:\n\n"
-	                           "%1\n\n"
-	                           "This typically happens for one of the following reasons:\n"
-	                           "- Outdated traineddata files are used.\n"
-	                           "- Auxiliary language data files are missing.\n"
-	                           "- Corrupt language data files.\n\n"
-	                           "Make sure your language data files are valid and compatible with tesseract %2.")).arg(captured).arg(tess.Version());
-	QMessageBox::critical(MAIN, _("Error"), errMsg);
-	longjmp(g_restore_point, SIGSEGV);
-}
-
 tesseract::TessBaseAPI Recognizer::initTesseract(const char* language, bool* ok) const {
+	// unfortunately tesseract creates deliberate aborts when an error occurs
+	std::signal(SIGABRT, MainWindow::tesseractCrash);
 	QByteArray current = setlocale(LC_ALL, NULL);
 	setlocale(LC_ALL, "C");
 	tesseract::TessBaseAPI tess;
-	// unfortunately tesseract creates deliberate segfaults when an error occurs
-	std::signal(SIGSEGV, tessCrashHandler);
-	pipe(g_pipe);
-	fflush(stderr);
-	int oldstderr = dup(fileno(stderr));
-	dup2(g_pipe[1], fileno(stderr));
-	int ret = -1;
-	int fault_code = setjmp(g_restore_point);
-	if(fault_code == 0) {
-		ret = tess.Init(nullptr, language);
-	} else {
-		ret = -1;
-	}
-	dup2(oldstderr, fileno(stderr));
-	std::signal(SIGSEGV, MainWindow::signalHandler);
+	int ret = tess.Init(nullptr, language);
 	setlocale(LC_NUMERIC, current.constData());
 
-	close(g_pipe[0]);
-	close(g_pipe[1]);
 	if(ok) {
 		*ok = ret != -1;
 	}
@@ -328,6 +297,7 @@ void Recognizer::updateLanguagesMenu() {
 	QAction* psmAction = new QAction(_("Page segmentation mode"), ui.menuLanguages);
 	psmAction->setMenu(psmMenu);
 	ui.menuLanguages->addAction(psmAction);
+	ui.menuLanguages->addAction(_("Character whitelist / blacklist..."), this, SLOT(manageCharacterLists()));
 
 
 	// Add installer item
@@ -383,6 +353,10 @@ void Recognizer::psmSelected(QAction* action) {
 	ConfigSettings::get<VarSetting<int>>("psm")->setValue(action->data().toInt());
 }
 
+void Recognizer::manageCharacterLists() {
+	m_charListDialog->exec();
+}
+
 QList<int> Recognizer::selectPages(bool& autodetectLayout) {
 	int nPages = MAIN->getDisplayer()->getNPages();
 
@@ -462,6 +436,12 @@ void Recognizer::recognize(const QList<int>& pages, bool autodetectLayout) {
 	if(ok) {
 		QString failed;
 		tess.SetPageSegMode(static_cast<tesseract::PageSegMode>(m_psmCheckGroup->checkedAction()->data().toInt()));
+		if(m_charListDialogUi.radioButtonWhitelist->isChecked()) {
+			tess.SetVariable("tessedit_char_whitelist", m_charListDialogUi.lineEditWhitelist->text().toLocal8Bit());
+		}
+		if(m_charListDialogUi.radioButtonBlacklist->isChecked()) {
+			tess.SetVariable("tessedit_char_blacklist", m_charListDialogUi.lineEditBlacklist->text().toLocal8Bit());
+		}
 		OutputEditor::ReadSessionData* readSessionData = MAIN->getOutputEditor()->initRead(tess);
 		ProgressMonitor monitor(pages.size());
 		MAIN->showProgress(&monitor);
