@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * Utils.cc
- * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2019 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -203,10 +203,7 @@ QByteArray Utils::download(QUrl url, QString& messages, int timeout) {
 
 QString Utils::getSpellingLanguage(const QString& lang) {
 	// If it is already a valid code, return it
-	if(lang.length() == 2) {
-		return lang;
-	}
-	if(QRegExp("^[a-z]{2}_[A-Z]{2}").indexIn(lang) != -1) {
+	if(QRegExp("[a-z]{2,}(_[A-Z]{2,})?").exactMatch(lang)) {
 		return lang;
 	}
 	// Treat the language as a tesseract lang spec and try to find a matching code
