@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * HOCROdtExporter.cc
- * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2019 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -295,8 +295,8 @@ void HOCROdtExporter::writeFontFaceDecls(QSet<QString>& families, const HOCRItem
 			families.insert(fontFamily);
 		}
 	} else {
-		for(const HOCRItem* item : item->children()) {
-			writeFontFaceDecls(families, item, writer);
+		for(const HOCRItem* child : item->children()) {
+			writeFontFaceDecls(families, child, writer);
 		}
 	}
 }
@@ -327,8 +327,8 @@ void HOCROdtExporter::writeFontStyles(QMap<QString, QMap<double, QString>>& styl
 			styles[fontKey].insert(item->fontSize(), styleName);
 		}
 	} else {
-		for(const HOCRItem* item : item->children()) {
-			writeFontStyles(styles, item, writer, counter);
+		for(const HOCRItem* child : item->children()) {
+			writeFontStyles(styles, child, writer, counter);
 		}
 	}
 }
