@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * HOCRPdfExporter.cc
- * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2019 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -432,6 +432,7 @@ HOCRPdfExporter::HOCRPdfExporter(const HOCRDocument* hocrdocument, const HOCRPag
 	ADD_SETTING(SpinSetting("pdffontsize", ui.spinBoxFontSize, 10));
 	ADD_SETTING(SpinSetting("pdffontscale", ui.spinFontScaling, 100));
 	ADD_SETTING(SwitchSetting("pdfuniformizelinespacing", ui.checkBoxUniformizeSpacing, false));
+	ADD_SETTING(SwitchSetting("pdfsanitizehyphens", ui.checkBoxSanitizeHyphens, true));
 	ADD_SETTING(SpinSetting("pdfpreservespaces", ui.spinBoxPreserve, 4));
 	ADD_SETTING(SwitchSetting("pdfpreview", ui.checkBoxPreview, false));
 	ADD_SETTING(ComboSetting("pdfexportpapersize", ui.comboBoxPaperSize));
@@ -553,15 +554,26 @@ bool HOCRPdfExporter::run(QString& filebasename) {
 			const HOCRPage* page = m_hocrdocument->page(i);
 			if(page->isEnabled()) {
 				QRect bbox = page->bbox();
+				QString sourceFile = page->sourceFile();
+				// If the source file is an image, its "resolution" is actually just the scale factor that was used for recognizing.
+				bool isImage = !sourceFile.endsWith(".pdf", Qt::CaseInsensitive) && !sourceFile.endsWith(".djvu", Qt::CaseInsensitive);
 				int sourceDpi = page->resolution();
+				int sourceScale = sourceDpi;
+				if(isImage) {
+					sourceDpi *= ui.spinBoxPaperSizeDpi->value() / 100;
+				}
+				// [pt] = 72 * [in]
+				// [in] = 1 / dpi * [px]
+				// => [pt] = 72 / dpi * [px]
+				double px2pt = (72.0 / sourceDpi);
+				double imgScale = double(outputDpi) / sourceDpi;
 				bool success = false;
-				QMetaObject::invokeMethod(this, "setSource", Qt::BlockingQueuedConnection, Q_RETURN_ARG(bool, success), Q_ARG(QString, page->sourceFile()), Q_ARG(int, page->pageNr()), Q_ARG(int, outputDpi), Q_ARG(double, page->angle()));
+				if(isImage) {
+					QMetaObject::invokeMethod(this, "setSource", Qt::BlockingQueuedConnection, Q_RETURN_ARG(bool, success), Q_ARG(QString, sourceFile), Q_ARG(int, page->pageNr()), Q_ARG(int, int(sourceScale * imgScale)), Q_ARG(double, page->angle()));
+				} else {
+					QMetaObject::invokeMethod(this, "setSource", Qt::BlockingQueuedConnection, Q_RETURN_ARG(bool, success), Q_ARG(QString, sourceFile), Q_ARG(int, page->pageNr()), Q_ARG(int, outputDpi), Q_ARG(double, page->angle()));
+				}
 				if(success) {
-					// [pt] = 72 * [in]
-					// [in] = 1 / dpi * [px]
-					// => [pt] = 72 / dpi * [px]
-					double px2pt = (72.0 / sourceDpi);
-					double imgScale = double(outputDpi) / sourceDpi;
 					if(paperSize == "source") {
 						pageWidth = bbox.width() * px2pt;
 						pageHeight = bbox.height() * px2pt;
@@ -571,7 +583,7 @@ bool HOCRPdfExporter::run(QString& filebasename) {
 					if(!painter->createPage(pageWidth, pageHeight, offsetX, offsetY, errMsg)) {
 						return false;
 					}
-					printChildren(*painter, page, pdfSettings, px2pt, imgScale);
+					printChildren(*painter, page, pdfSettings, px2pt, imgScale, double(sourceScale) / sourceDpi);
 					if(pdfSettings.overlay) {
 						QRect scaledRect(imgScale * bbox.left(), imgScale * bbox.top(), imgScale * bbox.width(), imgScale * bbox.height());
 						QRect printRect(bbox.left() * px2pt, bbox.top() * px2pt, bbox.width() * px2pt, bbox.height() * px2pt);
@@ -579,11 +591,11 @@ bool HOCRPdfExporter::run(QString& filebasename) {
 						QMetaObject::invokeMethod(this, "getSelection",  Qt::BlockingQueuedConnection, Q_RETURN_ARG(QImage, selection), Q_ARG(QRect, scaledRect));
 						painter->drawImage(printRect, selection, pdfSettings);
 					}
-					QMetaObject::invokeMethod(this, "setSource", Qt::BlockingQueuedConnection, Q_RETURN_ARG(bool, success), Q_ARG(QString, page->sourceFile()), Q_ARG(int, page->pageNr()), Q_ARG(int, sourceDpi), Q_ARG(double, page->angle()));
+					QMetaObject::invokeMethod(this, "setSource", Qt::BlockingQueuedConnection, Q_RETURN_ARG(bool, success), Q_ARG(QString, page->sourceFile()), Q_ARG(int, page->pageNr()), Q_ARG(int, sourceScale), Q_ARG(double, page->angle()));
 					painter->finishPage();
 				} else {
 					errMsg = _("Failed to render page %1").arg(page->title());
-					return false;;
+					return false;
 				}
 			}
 			monitor.increaseProgress();
@@ -675,13 +687,17 @@ HOCRPdfExporter::PDFSettings HOCRPdfExporter::getPdfSettings() const {
 	pdfSettings.preserveSpaceWidth = ui.spinBoxPreserve->value();
 	pdfSettings.overlay = ui.comboBoxOutputMode->currentIndex() == 1;
 	pdfSettings.detectedFontScaling = ui.spinFontScaling->value() / 100.;
+	pdfSettings.sanitizeHyphens = ui.checkBoxSanitizeHyphens->isChecked();
 	return pdfSettings;
 }
 
-void HOCRPdfExporter::printChildren(PDFPainter& painter, const HOCRItem* item, const PDFSettings& pdfSettings, double px2pu/*pixels to printer units*/, double imgScale) {
+void HOCRPdfExporter::printChildren(PDFPainter& painter, const HOCRItem* item, const PDFSettings& pdfSettings, double px2pu/*pixels to printer units*/, double imgScale, double fontScale) {
 	if(!item->isEnabled()) {
 		return;
 	}
+	if(pdfSettings.fontSize != -1) {
+		painter.setFontSize(pdfSettings.fontSize * fontScale);
+	}
 	QString itemClass = item->itemClass();
 	QRect itemRect = item->bbox();
 	int childCount = item->children().size();
@@ -701,7 +717,7 @@ void HOCRPdfExporter::printChildren(PDFPainter& painter, const HOCRItem* item, c
 				QRect wordRect = wordItem->bbox();
 				painter.setFontFamily(pdfSettings.fontFamily.isEmpty() ? wordItem->fontFamily() : pdfSettings.fontFamily, wordItem->fontBold(), wordItem->fontItalic());
 				if(pdfSettings.fontSize == -1) {
-					painter.setFontSize(wordItem->fontSize() * pdfSettings.detectedFontScaling);
+					painter.setFontSize(wordItem->fontSize() * pdfSettings.detectedFontScaling * fontScale);
 				}
 				// If distance from previous word is large, keep the space
 				if(wordRect.x() - prevWordRight > pdfSettings.preserveSpaceWidth * painter.getAverageCharWidth() / px2pu) {
@@ -709,6 +725,9 @@ void HOCRPdfExporter::printChildren(PDFPainter& painter, const HOCRItem* item, c
 				}
 				prevWordRight = wordRect.right();
 				QString text = wordItem->text();
+				if(iWord == nWords - 1 && pdfSettings.sanitizeHyphens) {
+					text.replace(QRegExp("[-\u2014]\\s*$"), "-");
+				}
 				double wordBaseline = (x - itemRect.x()) * baseline.first + baseline.second;
 				painter.drawText(x * px2pu, (y + wordBaseline) * px2pu, text);
 				x += painter.getTextWidth(text + " ") / px2pu;
@@ -724,10 +743,14 @@ void HOCRPdfExporter::printChildren(PDFPainter& painter, const HOCRItem* item, c
 			QRect wordRect = wordItem->bbox();
 			painter.setFontFamily(pdfSettings.fontFamily.isEmpty() ? wordItem->fontFamily() : pdfSettings.fontFamily, wordItem->fontBold(), wordItem->fontItalic());
 			if(pdfSettings.fontSize == -1) {
-				painter.setFontSize(wordItem->fontSize() * pdfSettings.detectedFontScaling);
+				painter.setFontSize(wordItem->fontSize() * pdfSettings.detectedFontScaling * fontScale);
 			}
 			double y = itemRect.bottom() + (wordRect.center().x() - itemRect.x()) * baseline.first + baseline.second;
-			painter.drawText(wordRect.x() * px2pu, y * px2pu, wordItem->text());
+			QString text = wordItem->text();
+			if(iWord == nWords - 1 && pdfSettings.sanitizeHyphens) {
+				text.replace(QRegExp("[-\u2014]\\s*$"), "-");
+			}
+			painter.drawText(wordRect.x() * px2pu, y * px2pu, text);
 		}
 	} else if(itemClass == "ocr_graphic" && !pdfSettings.overlay) {
 		QRect scaledItemRect(itemRect.left() * imgScale, itemRect.top() * imgScale, itemRect.width() * imgScale, itemRect.height() * imgScale);
@@ -737,7 +760,7 @@ void HOCRPdfExporter::printChildren(PDFPainter& painter, const HOCRItem* item, c
 		painter.drawImage(printRect, selection, pdfSettings);
 	} else {
 		for(int i = 0, n = item->children().size(); i < n; ++i) {
-			printChildren(painter, item->children()[i], pdfSettings, px2pu, imgScale);
+			printChildren(painter, item->children()[i], pdfSettings, px2pu, imgScale, fontScale);
 		}
 	}
 }
@@ -826,7 +849,8 @@ void HOCRPdfExporter::imageFormatChanged() {
 	QStandardItem* ccittItem = model->item(ccittIdx);
 	QStandardItem* jpegItem = model->item(jpegIdx);
 	if(format == QImage::Format_Mono) {
-		if(ui.comboBoxImageCompression->currentIndex() == jpegIdx) {
+		// for monochrome images, allow zip and ccitt4 (but not jpeg, unless QPrinter is forcing us)
+		if(ui.comboBoxImageCompression->currentIndex() == jpegIdx && ui.comboBoxBackend->itemData(ui.comboBoxBackend->currentIndex()).toInt() != BackendQPrinter) {
 			ui.comboBoxImageCompression->setCurrentIndex(zipIdx);
 		}
 		ccittItem->setFlags(ccittItem->flags() | Qt::ItemIsSelectable | Qt::ItemIsEnabled);
@@ -834,6 +858,7 @@ void HOCRPdfExporter::imageFormatChanged() {
 		ui.labelDithering->setEnabled(true);
 		ui.comboBoxDithering->setEnabled(true);
 	} else {
+		// for color and grayscale images, allow jpeg and zip (but not ccitt4)
 		if(ui.comboBoxImageCompression->currentIndex() == ccittIdx) {
 			ui.comboBoxImageCompression->setCurrentIndex(zipIdx);
 		}
@@ -871,14 +896,17 @@ void HOCRPdfExporter::paperSizeChanged() {
 		ui.lineEditPaperHeight->setEnabled(true);
 		ui.widgetPaperSize->setVisible(true);
 		ui.widgetPaperOrientation->setVisible(false);
+		ui.widgetPaperSizeDpi->setVisible(false);
 	} else if(paperSize == "source") {
 		ui.lineEditPaperWidth->setEnabled(true);
 		ui.lineEditPaperHeight->setEnabled(true);
 		ui.widgetPaperSize->setVisible(false);
 		ui.widgetPaperOrientation->setVisible(false);
+		ui.widgetPaperSizeDpi->setVisible(true);
 	} else {
 		ui.widgetPaperSize->setVisible(true);
 		ui.widgetPaperOrientation->setVisible(true);
+		ui.widgetPaperSizeDpi->setVisible(false);
 		ui.lineEditPaperWidth->setDisabled(true);
 		ui.lineEditPaperHeight->setDisabled(true);
 		PaperSize::Unit unit = static_cast<PaperSize::Unit>(ui.comboBoxPaperSizeUnit->itemData(ui.comboBoxPaperSizeUnit->currentIndex()).toInt());
