@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * OutputEditorHOCR.cc
- * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2019 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -440,9 +440,9 @@ void OutputEditorHOCR::showItemProperties(const QModelIndex& index, const QModel
 	}
 
 	// ocr_class:attr_key:attr_values
-	QMap<QString, QMap<QString, QSet<QString>>> occurences;
-	currentItem->getPropagatableAttributes(occurences);
-	for(auto it = occurences.begin(), itEnd = occurences.end(); it != itEnd; ++it) {
+	QMap<QString, QMap<QString, QSet<QString>>> occurrences;
+	currentItem->getPropagatableAttributes(occurrences);
+	for(auto it = occurrences.begin(), itEnd = occurrences.end(); it != itEnd; ++it) {
 		ui.tableWidgetProperties->insertRow(++row);
 		QTableWidgetItem* sectionItem = new QTableWidgetItem(it.key());
 		sectionItem->setFlags(sectionItem->flags() & ~(Qt::ItemIsEditable | Qt::ItemIsSelectable));
@@ -488,7 +488,7 @@ void OutputEditorHOCR::showItemProperties(const QModelIndex& index, const QModel
 QWidget* OutputEditorHOCR::createAttrWidget(const QModelIndex& itemIndex, const QString& attrName, const QString& attrValue, const QString& attrItemClass, bool multiple) {
 	static QMap<QString, QString> attrLineEdits = {
 		{"title:bbox", "\\d+\\s+\\d+\\s+\\d+\\s+\\d+"},
-		{"lang", "[a-z]{2}(?:_[A-Z]{2})?"},
+		{"lang", "[a-z]{2,}(?:_[A-Z]{2,})?"},
 		{"title:x_fsize", "\\d+"},
 		{"title:baseline", "[-+]?\\d+\\.?\\d*\\s[-+]?\\d+\\.?\\d*"}
 	};
@@ -546,6 +546,8 @@ void OutputEditorHOCR::bboxDrawn(const QRect& bbox, int action) {
 	if(!currentItem) {
 		return;
 	}
+	QMap<QString, QMap<QString, QSet<QString>>> propAttrs;
+	currentItem->getPropagatableAttributes(propAttrs);
 	QDomElement newElement;
 	if(action == DisplayerToolHOCR::ACTION_DRAW_GRAPHIC_RECT) {
 		newElement = doc.createElement("div");
@@ -562,6 +564,7 @@ void OutputEditorHOCR::bboxDrawn(const QRect& bbox, int action) {
 	} else if(action == DisplayerToolHOCR::ACTION_DRAW_LINE_RECT) {
 		newElement = doc.createElement("span");
 		newElement.setAttribute("class", "ocr_line");
+		QSet<QString> propLineBaseline = propAttrs["ocrx_line"]["baseline"];
 		// Tesseract does as follows:
 		// row_height = x_height + ascenders - descenders
 		// font_pt_size = row_height * 72 / dpi (72 = pointsPerInch)
@@ -571,7 +574,7 @@ void OutputEditorHOCR::bboxDrawn(const QRect& bbox, int action) {
 		titleAttrs["x_ascenders"] = QString("%1").arg(0.25 * bbox.height());
 		titleAttrs["x_descenders"] = QString("%1").arg(0.25 * bbox.height());
 		titleAttrs["x_size"] = QString("%1").arg(bbox.height());
-		titleAttrs["baseline"] = QString("0 0");
+		titleAttrs["baseline"] = propLineBaseline.size() == 1 ? *propLineBaseline.begin() : QString("0 0");
 		newElement.setAttribute("title", HOCRItem::serializeAttrGroup(titleAttrs));
 	} else if(action == DisplayerToolHOCR::ACTION_DRAW_WORD_RECT) {
 		QString text = QInputDialog::getText(m_widget, _("Add Word"), _("Enter word:"));
@@ -580,20 +583,26 @@ void OutputEditorHOCR::bboxDrawn(const QRect& bbox, int action) {
 		}
 		newElement = doc.createElement("span");
 		newElement.setAttribute("class", "ocrx_word");
-		newElement.setAttribute("lang", m_spell.getLanguage());
+		QMap<QString, QSet<QString>> propWord = propAttrs["ocrx_word"];
+
+		newElement.setAttribute("lang", propWord["lang"].size() == 1 ? *propWord["lang"].begin() : m_spell.getLanguage());
 		QMap<QString, QString> titleAttrs;
 		titleAttrs["bbox"] = QString("%1 %2 %3 %4").arg(bbox.left()).arg(bbox.top()).arg(bbox.right()).arg(bbox.bottom());
-		titleAttrs["x_font"] = QFont().family();
-		titleAttrs["x_fsize"] = QString("%1").arg(qRound(bbox.height() * 72. / currentItem->page()->resolution()));
 		titleAttrs["x_wconf"] = "100";
+		titleAttrs["x_font"] = propWord["title:x_font"].size() == 1 ?
+		                       *propWord["title:x_font"].begin() : QFont().family();
+		titleAttrs["x_fsize"] = propWord["title:x_fsize"].size() == 1 ?
+		                        *propWord["title:x_fsize"].begin() : QString("%1").arg(qRound(bbox.height() * 72. / currentItem->page()->resolution()));
 		newElement.setAttribute("title", HOCRItem::serializeAttrGroup(titleAttrs));
+		if (propWord["bold"].size() == 1) { newElement.setAttribute("bold", *propWord["bold"].begin()); }
+		if (propWord["italic"].size() == 1) { newElement.setAttribute("italic", *propWord["italic"].begin()); }
 		newElement.appendChild(doc.createTextNode(text));
 	} else {
 		return;
 	}
 	QModelIndex index = m_document->addItem(current, newElement);
 	if(index.isValid()) {
-		ui.treeViewHOCR->selectionModel()->setCurrentIndex(index, QItemSelectionModel::ClearAndSelect);
+		ui.treeViewHOCR->selectionModel()->setCurrentIndex(index, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
 	}
 }
 
@@ -625,17 +634,17 @@ void OutputEditorHOCR::showTreeWidgetContextMenu(const QPoint& point) {
 		bool pages = firstItem->itemClass() == "ocr_page";
 		bool sameClass = classes.size() == 1;
 
-		QAction* mergeAction = nullptr;
-		QAction* splitAction = nullptr;
-		QAction* swapAction = nullptr;
+		QAction* actionMerge = nullptr;
+		QAction* actionSplit = nullptr;
+		QAction* actionSwap = nullptr;
 		if(consecutive && !graphics && !pages && sameClass) { // Merging allowed
-			mergeAction = menu.addAction(_("Merge"));
+			actionMerge = menu.addAction(_("Merge"));
 			if(firstItem->itemClass() != "ocr_carea") {
-				splitAction = menu.addAction(_("Split from parent"));
+				actionSplit = menu.addAction(_("Split from parent"));
 			}
 		}
 		if(nIndices == 2) { // Swapping allowed
-			swapAction = menu.addAction(_("Swap"));
+			actionSwap = menu.addAction(_("Swap"));
 		}
 
 		QAction* clickedAction = menu.exec(ui.treeViewHOCR->mapToGlobal(point));
@@ -644,12 +653,12 @@ void OutputEditorHOCR::showTreeWidgetContextMenu(const QPoint& point) {
 		}
 		ui.treeViewHOCR->selectionModel()->blockSignals(true);
 		QModelIndex newIndex;
-		if(clickedAction == mergeAction) {
+		if(clickedAction == actionMerge) {
 			newIndex = m_document->mergeItems(indices.first().parent(), rows.first(), rows.last());
-		} else if(clickedAction == splitAction) {
+		} else if(clickedAction == actionSplit) {
 			newIndex = m_document->splitItem(indices.first().parent(), rows.first(), rows.last());
 			expandCollapseChildren(newIndex, true);
-		} else if(clickedAction == swapAction) {
+		} else if(clickedAction == actionSwap) {
 			newIndex = m_document->swapItems(indices.first().parent(), rows.first(), rows.last());
 		}
 		if(newIndex.isValid()) {
@@ -673,10 +682,10 @@ void OutputEditorHOCR::showTreeWidgetContextMenu(const QPoint& point) {
 	QAction* actionAddLine = nullptr;
 	QAction* actionAddWord = nullptr;
 	QAction* actionSplit = nullptr;
-	QAction* addWordAction = nullptr;
-	QAction* ignoreWordAction = nullptr;
+	QAction* actionDictAddWord = nullptr;
+	QAction* actionDictIgnoreWord = nullptr;
 	QList<QAction*> setTextActions;
-	QAction* actionRemoveItem = nullptr;
+	QAction* actionRemove = nullptr;
 	QAction* actionExpand = nullptr;
 	QAction* actionCollapse = nullptr;
 	QString itemClass = item->itemClass();
@@ -691,24 +700,21 @@ void OutputEditorHOCR::showTreeWidgetContextMenu(const QPoint& point) {
 		actionAddWord = menu.addAction(_("Add word"));
 	} else if(itemClass == "ocrx_word") {
 		QString prefix, suffix, trimmedWord = HOCRItem::trimmedWord(item->text(), &prefix, &suffix);
-		QString spellLang = item->lang();
-		bool haveLanguage = true;
-		if(m_spell.getLanguage() != spellLang) {
-			haveLanguage = m_spell.setLanguage(spellLang);
+		QStringList suggestions;
+		bool valid = m_document->checkItemSpelling(index, &suggestions, 16);
+		for(const QString& suggestion : suggestions) {
+			setTextActions.append(menu.addAction(prefix + suggestion + suffix));
 		}
-		if(!trimmedWord.isEmpty() && haveLanguage) {
-			for(const QString& suggestion : m_spell.getSpellingSuggestions(trimmedWord)) {
-				setTextActions.append(menu.addAction(prefix + suggestion + suffix));
-			}
+		if(!trimmedWord.isEmpty()) {
 			if(setTextActions.isEmpty()) {
 				menu.addAction(_("No suggestions"))->setEnabled(false);
 			}
-			if(!m_spell.checkWord(trimmedWord)) {
+			if(!valid) {
 				menu.addSeparator();
-				addWordAction = menu.addAction(_("Add to dictionary"));
-				addWordAction->setData(trimmedWord);
-				ignoreWordAction = menu.addAction(_("Ignore word"));
-				ignoreWordAction->setData(trimmedWord);
+				actionDictAddWord = menu.addAction(_("Add to dictionary"));
+				actionDictAddWord->setData(trimmedWord);
+				actionDictIgnoreWord = menu.addAction(_("Ignore word"));
+				actionDictIgnoreWord->setData(trimmedWord);
 			}
 		}
 	}
@@ -718,8 +724,8 @@ void OutputEditorHOCR::showTreeWidgetContextMenu(const QPoint& point) {
 	if(itemClass == "ocr_par" || itemClass == "ocr_line" || itemClass == "ocrx_word") {
 		actionSplit = menu.addAction(_("Split from parent"));
 	}
-	actionRemoveItem = menu.addAction(_("Remove"));
-	actionRemoveItem->setShortcut(QKeySequence(Qt::Key_Delete));
+	actionRemove = menu.addAction(_("Remove"));
+	actionRemove->setShortcut(QKeySequence(Qt::Key_Delete));
 	if(m_document->rowCount(index) > 0) {
 		actionExpand = menu.addAction(_("Expand all"));
 		actionCollapse = menu.addAction(_("Collapse all"));
@@ -739,11 +745,11 @@ void OutputEditorHOCR::showTreeWidgetContextMenu(const QPoint& point) {
 		m_tool->setAction(DisplayerToolHOCR::ACTION_DRAW_LINE_RECT);
 	} else if(clickedAction == actionAddWord) {
 		m_tool->setAction(DisplayerToolHOCR::ACTION_DRAW_WORD_RECT);
-	} else if(clickedAction == addWordAction) {
-		m_spell.addWordToDictionary(addWordAction->data().toString());
+	} else if(clickedAction == actionDictAddWord) {
+		m_spell.addWordToDictionary(actionDictAddWord->data().toString());
 		m_document->recheckSpelling();
-	} else if(clickedAction == ignoreWordAction) {
-		m_spell.ignoreWord(ignoreWordAction->data().toString());
+	} else if(clickedAction == actionDictIgnoreWord) {
+		m_spell.ignoreWord(actionDictIgnoreWord->data().toString());
 		m_document->recheckSpelling();
 	} else if(setTextActions.contains(clickedAction)) {
 		m_document->setData(index, clickedAction->text(), Qt::EditRole);
@@ -751,7 +757,7 @@ void OutputEditorHOCR::showTreeWidgetContextMenu(const QPoint& point) {
 		QModelIndex newIndex = m_document->splitItem(index.parent(), index.row(), index.row());
 		ui.treeViewHOCR->selectionModel()->setCurrentIndex(newIndex, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
 		expandCollapseChildren(newIndex, true);
-	} else if(clickedAction == actionRemoveItem) {
+	} else if(clickedAction == actionRemove) {
 		m_document->removeItem(ui.treeViewHOCR->selectionModel()->currentIndex());
 	} else if(clickedAction == actionExpand) {
 		expandCollapseChildren(index, true);
@@ -833,7 +839,10 @@ bool OutputEditorHOCR::save(const QString& filename) {
 		QMessageBox::critical(MAIN, _("Failed to save output"), _("Check that you have writing permissions in the selected folder."));
 		return false;
 	}
+	QByteArray current = setlocale(LC_ALL, NULL);
+	setlocale(LC_ALL, "C");
 	tesseract::TessBaseAPI tess;
+	setlocale(LC_ALL, current.constData());
 	QString header = QString(
 	                     "<!DOCTYPE html>\n"
 	                     "<html>\n"
@@ -907,7 +916,7 @@ void OutputEditorHOCR::setLanguage(const Config::Lang& lang) {
 	m_document->setDefaultLanguage(lang.code);
 }
 
-void OutputEditorHOCR::onVisibilityChanged(bool /*visibile*/) {
+void OutputEditorHOCR::onVisibilityChanged(bool /*visible*/) {
 	ui.searchFrame->hideSubstitutionsManager();
 }
 
@@ -1064,7 +1073,9 @@ void OutputEditorHOCR::drawPreview(QPainter& painter, const HOCRItem* item) {
 			}
 			const QRect& wordRect = wordItem->bbox();
 			QFont font;
-			font.setFamily(wordItem->fontFamily());
+			if(!wordItem->fontFamily().isEmpty()) {
+				font.setFamily(wordItem->fontFamily());
+			}
 			font.setBold(wordItem->fontBold());
 			font.setItalic(wordItem->fontItalic());
 			font.setPointSizeF(wordItem->fontSize());
