@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * main.cc
- * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2019 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -68,8 +68,9 @@ int main (int argc, char* argv[]) {
 	QWidget* window;
 	if(argc >= 3 && std::strcmp("crashhandle", argv[1]) == 0) {
 		int pid = std::atoi(argv[2]);
-		QString savefile = argc >= 4 ? argv[3] : "";
-		window = new CrashHandler(pid, savefile);
+		int tesseractCrash = std::atoi(argv[3]);
+		QString savefile = argc >= 5 ? argv[4] : "";
+		window = new CrashHandler(pid, tesseractCrash, savefile);
 	} else if(argc >= 2 && std::strcmp("tessdatadir", argv[1]) == 0) {
 		Config::openTessdataDir();
 		return 0;
